      MODULE f_bsllen
      CONTAINS

C*
      FUNCTION BSLLEN(STN1,STN2)
CC
CC NAME       :  BSLLEN
CC
CC
CC PURPOSE    :  COMPUTE BASELINE LENGTH BETWEEN TWO STATIONS.
CC
CC
CC PARAMETERS :
CC         IN :  STN1   : NAME OF THE FIRST STATION             CH*16
CC               STN2   : NAME OF THE SECOND STATION            CH*16
CC        OUT :  BSLLEN : BASELINE LENGTH IN [M]                R*8
CC
CC REMARKS    :  ---
CC
CC AUTHOR     :  S.FANKHAUSER
CC
CC VERSION    :  3.6
CC
CC CREATED    :  09-OCT-95
CC
CC CHANGES    :  19-MAR-96 : EB: CHECK IF COORDINATE FILE IS AVAILABLE
CC               23-JUN-05 : MM: IMPLICIT NONE AND DECLARATIONS ADDED
CC               15-FEB-11 : RD: GETSTA IS USED AS A MODULE NOW
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1987     UNIVERSITY OF BERN
CC               SWITZERLAND
CC
C*
      USE s_getsta
      USE s_gtflna
      IMPLICIT NONE
C
C DECLARATIONS INSTEAD OF IMPLICIT
C --------------------------------
      INTEGER*4 I     , ID2   , IRC   , NSTAT
C
      REAL*8    BSLHLP, BSLLEN, D11   , D7    , D8
C
CCC       IMPLICIT REAL*8 (A-H,O-Z)
CCC       IMPLICIT INTEGER*4 (I-N)
C
C GLOBAL DECLARATIONS
C -------------------
      CHARACTER*16 STN1,STN2
C
C
C LOCAL DECLARATIONS
C ------------------
      CHARACTER*32 COOFIL
      CHARACTER*16 STA(4),D6
      INTEGER*4    D1(4),ID3(4)
      REAL*8       XSTAT(3,4),D4(3,4),D5(3,4),D9(3),D10(3)
C
C CHECK IF COORDINATE FILE IS AVAILABLE
C
      CALL GTFLNA(0,'COORD  ',COOFIL,IRC)
      IF (IRC.EQ.0) THEN
C
C GET COORDINATES OF BOTH STATIONS
C --------------------------------
        STA(1)=STN1
        STA(2)=STN2
        NSTAT=2
        CALL GETSTA(NSTAT,STA,D1,ID2,ID3,XSTAT,
     1              D4,D5,D6,D7,D8,D9,D10,D11)
C
C COMPUTE BASELINE LENGTH
C -----------------------
        BSLHLP=0.D0
        DO 10 I=1,3
          BSLHLP = BSLHLP + (XSTAT(I,1)-XSTAT(I,2))**2
10      CONTINUE
C
        BSLLEN = DSQRT(BSLHLP)
      ELSE
        BSLLEN = 0.D0
      ENDIF
C
      RETURN
      END FUNCTION

      END MODULE
